@@ -21,10 +21,20 @@
 
 #include <QWidget>
 #include "termwidget.h"
+#include "terminalconfig.h"
+#include "dbusaddressable.h"
 class QSplitter;
 
 
 
+typedef enum NavigationDirection {
+    Left,
+    Right,
+    Top,
+    Bottom
+} NavigationDirection;
+
+
 /*! \brief TermWidget group/session manager.
 
 This widget (one per TabWidget tab) is a "proxy" widget beetween TabWidget and
@@ -34,11 +44,14 @@ for TabWidget - with its signals and slots.
 Splitting and collapsing of TermWidgets is done here.
 */
 class TermWidgetHolder : public QWidget
+#ifdef HAVE_QDBUS
+    , public DBusAddressable
+#endif
 {
     Q_OBJECT
 
     public:
-        TermWidgetHolder(const QString & wdir, const QString & shell=QString(), QWidget * parent=0);
+        TermWidgetHolder(TerminalConfig &cfg, QWidget * parent=0);
         ~TermWidgetHolder();
 
         void propertiesChanged();
@@ -50,20 +63,30 @@ class TermWidgetHolder : public QWidget
         void zoomOut(uint step);
 
         TermWidget* currentTerminal();
+        TermWidget* split(TermWidget * term, Qt::Orientation orientation, TerminalConfig cfg);
+
+        #ifdef HAVE_QDBUS
+        QDBusObjectPath getActiveTerminal();
+        QList<QDBusObjectPath> getTerminals();
+        QDBusObjectPath getWindow();
+        void closeTab();
+        #endif
+
 
     public slots:
         void splitHorizontal(TermWidget * term);
         void splitVertical(TermWidget * term);
         void splitCollapse(TermWidget * term);
         void setWDir(const QString & wdir);
-        void switchNextSubterminal();
-        void switchPrevSubterminal();
+        void directionalNavigation(NavigationDirection dir);
         void clearActiveTerminal();
+        void onTermTitleChanged(QString title, QString icon) const;
 
     signals:
         void finished();
         void lastTerminalClosed();
         void renameSession();
+        void termTitleChanged(QString title, QString icon) const;
 
     private:
         QString m_wdir;
@@ -71,7 +94,7 @@ class TermWidgetHolder : public QWidget
         TermWidget * m_currentTerm;
 
         void split(TermWidget * term, Qt::Orientation orientation);
-        TermWidget * newTerm(const QString & wdir=QString(), const QString & shell=QString());
+        TermWidget * newTerm(TerminalConfig &cfg);
 
     private slots:
         void setCurrentTerminal(TermWidget* term);
